import json
import os
from definitions import *
from exps import *
from helpers import fresh_request_uid
from redis import StrictRedis


class RestfulRedisClient(object):

  def __init__(self, url=None, channel=default_channel):
    url = url or os.environ.get('REDIS_URL') or default_url
    self.redis = StrictRedis.from_url(url=url)
    self.channel = channel

  def request(self, data={}, timeout=60):
    # Create a uid for this specific request
    request_uid = fresh_request_uid()

    # Wrap custom data into payload, along with request uid
    payload = {
      'data': data,
      'request_uid': request_uid
    }

    # Push the request data to the request queue
    self.redis.rpush(self.channel, json.dumps(payload))

    # Wait for response
    resp = self.redis.blpop(request_uid, timeout=timeout)

    # Response must exist or it's a timeout error
    if resp is None:
      raise RequestTimeout()

    try:
      # Attempt to parse JSON response
      parsed_resp = json.loads(resp[1])
    except:
      raise JsonParseException()

    # Delete request key
    self.redis.delete(request_uid)

    return parsed_resp


import json
import os
from definitions import *
from redis import StrictRedis
from exps import *


class RestfulRedisServer(object):

  def __init__(self, target, url=None, channel=default_channel):
    url = url or os.environ.get('REDIS_URL') or default_url
    self.redis = StrictRedis.from_url(url=url)
    self.channel = channel
    self.target = target

  def run(self):
    while True:
      # Block infinitely until request received
      request = self.redis.blpop(self.channel)

      try:
        # Attempt to parse JSON request data
        payload = json.loads(request[1]) or {}
      except:
        raise JsonParseException(request)

      # Get request uid and ensure its presence
      request_uid = payload.get('request_uid')

      if not request_uid:
        raise InvalidRequestException(payload)

      # Call target function to get JSON response
      resp = self.target(payload.get('data', {}))

      # Push the JSON response to channel=request_uid
      self.redis.rpush(request_uid, json.dumps(resp))


class JsonParseException(BaseException):
  def __init__(self, request):
    self.message = 'Error parsing JSON: {}'.format(request)


class InvalidRequestException(BaseException):
  def __init__(self, body):
    self.message = 'Invalid request body received: {}. \'request_uid\' key required.'.format(body)


class RequestTimeout(BaseException):
  def __init__(self):
    self.message = 'Request timed out.'


from setuptools import setup, find_packages

setup(name='restful_redis',
      version='0.0.1',
      description='Redis queues for synchronous server2server ops',
      url='https://github.com/whittlbc/restful_redis',
      author='Ben Whittle',
      author_email='benwhittle31@gmail.com',
      license='MIT.',
      packages=find_packages(),
      install_requires=[
        'redis==2.10.6'
      ],
      zip_safe=False)